# -*- coding: utf-8 -*-

import numpy as np
import pytest

from exoplanet_core.numpy import ops as nops
from exoplanet_core.testing import (
    check_contact_points,
    get_mean_and_true_anomaly,
)

fallback = pytest.importorskip("aesara_theano_fallback")
ops = pytest.importorskip("exoplanet_core.pymc.ops")

aesara = fallback.aesara


def compare_to_numpy(nop, op, *args):
    expected = nop(*args)
    computed = op(*args)
    np.testing.assert_allclose(expected, computed)


@pytest.fixture
def kepler_data():
    e = np.linspace(0, 1, 500)[:-1]
    E = np.linspace(-300, 300, 1001)
    e = e[None, :] + np.zeros((len(E), len(e)))
    E = E[:, None] + np.zeros_like(e)
    M, f = get_mean_and_true_anomaly(e, E)
    return M, e, f


def test_kepler(kepler_data):
    M, e, f = kepler_data
    sinf, cosf = aesara.function([], ops.kepler(M, e))()
    assert np.all(np.isfinite(cosf))
    np.testing.assert_allclose(cosf, np.cos(f), atol=1e-7)
    assert np.all(np.isfinite(sinf))
    np.testing.assert_allclose(sinf, np.sin(f), atol=1e-7)


def test_kepler_grad(kepler_data):
    try:
        verify_grad = aesara.tensor.verify_grad
    except AttributeError:
        from aesara.gradient import verify_grad

    M, e, f = kepler_data
    np.random.seed(1324)
    ind = 100
    verify_grad(
        lambda *x: ops.kepler(*x)[0],
        [M[:, ind], e[:, ind]],
        rng=np.random,
        eps=1e-8,
    )
    verify_grad(
        lambda *x: ops.kepler(*x)[1],
        [M[:, ind], e[:, ind]],
        rng=np.random,
        eps=1e-8,
    )


@pytest.fixture
def limbdark_data():
    r = np.linspace(0.01, 1.5, 5)
    b = np.linspace(-1, 1, 500)[:, None] * (1 + r[None, :])
    return (b, r[None, :] + np.zeros_like(b))


def test_quad_solution_vector(limbdark_data):
    b_ = aesara.tensor.dmatrix()
    r_ = aesara.tensor.dmatrix()
    func = aesara.function([b_, r_], ops.quad_solution_vector(b_, r_))
    compare_to_numpy(nops.quad_solution_vector, func, *limbdark_data)


def test_quad_solution_vector_grad(limbdark_data):
    try:
        verify_grad = aesara.tensor.verify_grad
    except AttributeError:
        from aesara.gradient import verify_grad

    # The numerical estimate is bad at discontinuities
    eps = 1e-7
    b, r = limbdark_data
    m = np.abs(b) > 2 * eps
    m &= np.abs(np.abs(b) - (1 - r)) > 2 * eps
    m &= np.abs(np.abs(b) - (1 + r)) > 2 * eps

    verify_grad(ops.quad_solution_vector, (b[m], r[m]), rng=np.random, eps=eps)


@pytest.mark.parametrize("a", [5.0, 12.1234, 20000.0])
@pytest.mark.parametrize("L", [0.7, 1.0, 1.5])
def test_contact_points(a, L):
    args = [aesara.tensor.dscalar() for _ in range(7)]
    func = aesara.function(args, ops.contact_points(*args))

    es = np.linspace(0, 1, 25)[:-1]
    ws = np.linspace(-np.pi, np.pi, 51)
    bs = np.linspace(0, 1 - 1e-5, 5)
    for bi, b in enumerate(bs):
        for ei, e in enumerate(es):
            for wi, w in enumerate(ws):
                check_contact_points(func, L, a, b, e, w)



# -*- coding: utf-8 -*-

import numpy as np

from exoplanet_core import driver
from exoplanet_core.testing import get_mean_and_true_anomaly


def check_kepler(e, E):
    M, f = get_mean_and_true_anomaly(e, E)
    cosf0, sinf0 = np.nan + np.zeros_like(M), np.nan + np.zeros_like(M)
    driver.solve_kepler(M, e, sinf0, cosf0)
    assert np.all(np.isfinite(cosf0))
    np.testing.assert_allclose(cosf0, np.cos(f), atol=1e-7)
    assert np.all(np.isfinite(sinf0))
    np.testing.assert_allclose(sinf0, np.sin(f), atol=1e-7)


def test_kepler():
    e = np.linspace(0, 1, 500)[:-1]
    E = np.linspace(-300, 300, 1001)
    e = e[None, :] + np.zeros((len(E), len(e)))
    E = E[:, None] + np.zeros_like(e)
    check_kepler(e, E)


def test_kepler_edge_cases():
    E = np.array([0.0, 2 * np.pi, -226.2, -170.4])
    e = (1 - 1e-6) * np.ones_like(E)
    e[-1] = 0.9939879759519037
    check_kepler(e, E)


def test_kepler_at_pi():
    e = np.linspace(0, 1.0, 100)[:-1]
    E = np.pi + np.zeros_like(e)
    check_kepler(e, E)



# -*- coding: utf-8 -*-

import numpy as np
import pytest

from exoplanet_core import core
from exoplanet_core.numpy import ops

try:
    import kepler
except ImportError:
    kepler = None

try:
    import radvel
except ImportError:
    radvel = None

try:
    import exoplanet
except ImportError:
    exoplanet = None
else:
    from packaging import version

    if version.parse(exoplanet.__version__) != version.parse("0.4.5"):
        exoplanet = None

try:
    import starry
except ImportError:
    starry = None

try:
    import batman
except ImportError:
    batman = None


@pytest.fixture(scope="module", params=[0.0, 0.25, 0.5, 0.75, 0.95])
def kepler_args(request):
    N = 10000
    return np.linspace(-100, 100, N), request.param + np.zeros(N)


@pytest.mark.benchmark(group="kepler")
def test_numpy_kepler_benchmark(benchmark, kepler_args):
    benchmark(ops.kepler, *kepler_args)


@pytest.mark.skipif(kepler is None, reason="kepler.py is not installed")
@pytest.mark.benchmark(group="kepler")
def test_kepler_dot_py_kepler_benchmark(benchmark, kepler_args):
    benchmark(kepler.kepler, *kepler_args)


@pytest.mark.skipif(radvel is None, reason="radvel is not installed")
@pytest.mark.benchmark(group="kepler")
def test_radvel_kepler_benchmark(benchmark, kepler_args):
    M, ecc = kepler_args
    ecc = ecc[0]

    @benchmark
    def _radvel_kepler():
        E = radvel._kepler.kepler_array(M, ecc)
        cosE, sinE = np.cos(E), np.sin(E)
        denom = 1 + cosE
        tanf2 = np.sqrt((1 + ecc) / (1 - ecc)) * sinE / denom
        tanf2_2 = tanf2 * tanf2
        denom = 1 / (1 + tanf2_2)
        cosf = (1 - tanf2_2) * denom
        sinf = 2 * tanf2 * denom
        return sinf, cosf


@pytest.fixture(scope="module", params=[0.01, 0.1, 0.9])
def limbdark_args(request):
    N = 10000
    u1 = 0.2
    u2 = 0.3
    b = np.linspace(-1.0 - request.param, 1.0 + request.param, N)
    r = request.param + np.zeros_like(b)
    return u1, u2, b, r


@pytest.mark.benchmark(group="limbdark")
def test_quad_limbdark_benchmark(benchmark, limbdark_args):
    benchmark(core.quad_limbdark_light_curve, *limbdark_args)


@pytest.mark.skipif(exoplanet is None, reason="old exoplanet is not installed")
@pytest.mark.benchmark(group="limbdark")
def test_quad_limbdark_exoplanet_benchmark(benchmark, limbdark_args):
    u1, u2, b, r = limbdark_args
    ld = exoplanet.theano_ops.driver.SimpleLimbDark()

    @benchmark
    def _exoplanet():
        ld.set_u([-1, u1, u2])
        return ld.apply(b, r)


@pytest.mark.skipif(starry is None, reason="starry is not installed")
@pytest.mark.benchmark(group="limbdark", warmup=True)
def test_quad_limbdark_starry_benchmark(benchmark, limbdark_args):
    import aesara_theano_fallback.tensor as tt
    from aesara_theano_fallback import aesara

    u1, u2, b, r = limbdark_args
    args = (u1, u2, b, r[0])

    u1_ = tt.dscalar()
    u2_ = tt.dscalar()
    b_ = tt.dvector()
    r_ = tt.dscalar()
    m = starry.Map(udeg=2)
    m[1] = u1_
    m[2] = u2_
    func = aesara.function([u1_, u2_, b_, r_], m.flux(xo=b_, ro=r_) - 1)
    func(*args)
    benchmark(func, *args)


@pytest.mark.skipif(batman is None, reason="batman is not installed")
@pytest.mark.benchmark(group="limbdark")
def test_quad_limbdark_batman_benchmark(benchmark, limbdark_args):
    u1, u2, b, r = limbdark_args

    @benchmark
    def _batman():
        return batman._quadratic_ld._quadratic_ld(b, r[0], u1, u2, 1) - 1



# -*- coding: utf-8 -*-

import numpy as np
import pytest

from exoplanet_core.numpy import ops as nops
from exoplanet_core.testing import (
    check_contact_points,
    get_mean_and_true_anomaly,
)

jax = pytest.importorskip("jax")
jnp = pytest.importorskip("jax.numpy")
ops = pytest.importorskip("exoplanet_core.jax.ops")

try:
    from jax.lib import xla_bridge
except ImportError:
    ON_GPU = False
else:
    ON_GPU = xla_bridge.get_backend().platform == "gpu"


def compare_to_numpy(nop, op, *args):
    expected = nop(*args)
    computed = op(*args)
    np.testing.assert_allclose(expected, computed)


@pytest.fixture
def kepler_data():
    e = jnp.linspace(0, 1, 500)[:-1]
    E = jnp.linspace(-300, 300, 1001)
    e = e[None, :] + jnp.zeros((len(E), len(e)))
    E = E[:, None] + jnp.zeros_like(e)
    M, f = get_mean_and_true_anomaly(e, E)
    return M, e, f


def test_kepler(kepler_data):
    M, e, f = kepler_data
    sinf, cosf = ops.kepler(M, e)
    assert np.all(np.isfinite(cosf))
    np.testing.assert_allclose(cosf, np.cos(f), atol=1e-7)
    assert np.all(np.isfinite(sinf))
    np.testing.assert_allclose(sinf, np.sin(f), atol=1e-7)


def test_kepler_jit(kepler_data):
    M, e, f = kepler_data
    sinf, cosf = jax.jit(ops.kepler)(M, e)
    assert np.all(np.isfinite(cosf))
    np.testing.assert_allclose(cosf, np.cos(f), atol=1e-7)
    assert np.all(np.isfinite(sinf))
    np.testing.assert_allclose(sinf, np.sin(f), atol=1e-7)


def test_kepler_grad(kepler_data):
    from jax.test_util import check_grads

    M, e, f = kepler_data
    m = (0.01 < e) & (e < 0.9)
    check_grads(lambda *args: ops.kepler(*args)[0], [M[m], e[m]], 2, eps=1e-6)
    check_grads(lambda *args: ops.kepler(*args)[1], [M[m], e[m]], 2, eps=1e-6)


@pytest.fixture
def limbdark_data():
    r = np.linspace(0.01, 1.5, 5)
    b = np.linspace(-1, 1, 500)[:, None] * (1 + r[None, :])
    return (b, r[None, :] + np.zeros_like(b))


def test_quad_solution_vector(limbdark_data):
    compare_to_numpy(
        nops.quad_solution_vector, ops.quad_solution_vector, *limbdark_data
    )


def test_quad_solution_vector_jit(limbdark_data):
    compare_to_numpy(
        nops.quad_solution_vector,
        jax.jit(ops.quad_solution_vector),
        *limbdark_data
    )


def test_quad_solution_vector_grad(limbdark_data):
    from jax.test_util import check_grads

    # The numerical estimate is bad at discontinuities
    eps = 1e-7
    b, r = limbdark_data
    m = np.abs(b) > 2 * eps
    m &= np.abs(np.abs(b) - (1 - r)) > 2 * eps
    m &= np.abs(np.abs(b) - (1 + r)) > 2 * eps

    check_grads(ops.quad_solution_vector, (b[m], r[m]), 1, eps=eps)


@pytest.mark.skipif(ON_GPU, reason="not implemented on GPU")
@pytest.mark.parametrize("a", [5.0, 12.1234, 20000.0])
@pytest.mark.parametrize("L", [0.7, 1.0, 1.5])
def test_contact_points(a, L):
    es = np.linspace(0, 1, 25)[:-1]
    ws = np.linspace(-np.pi, np.pi, 51)
    bs = np.linspace(0, 1 - 1e-5, 5)
    for bi, b in enumerate(bs):
        for ei, e in enumerate(es):
            for wi, w in enumerate(ws):
                check_contact_points(ops.contact_points, L, a, b, e, w)



# -*- coding: utf-8 -*-

import numpy as np
import pytest

from exoplanet_core import core


def test_compare_starry():
    starry = pytest.importorskip("starry")
    u1 = 0.2
    u2 = 0.3
    b = np.linspace(-1.5, 1.5, 100)
    r = 0.1 + np.zeros_like(b)

    m = starry.Map(udeg=2)
    m[1:] = [u1, u2]
    expected = m.flux(xo=b, ro=r[0]).eval() - 1

    computed = core.quad_limbdark_light_curve(u1, u2, b, r)
    np.testing.assert_allclose(computed, expected, atol=1e-12)


def test_compare_batman():
    batman = pytest.importorskip("batman")
    u1 = 0.2
    u2 = 0.3
    b = np.linspace(-1.5, 1.5, 100)
    r = 0.1 + np.zeros_like(b)

    expected = batman._quadratic_ld._quadratic_ld(b, r[0], u1, u2, 1) - 1

    computed = core.quad_limbdark_light_curve(u1, u2, b, r)
    np.testing.assert_allclose(computed, expected, atol=1e-8)



# -*- coding: utf-8 -*-

__all__ = ["kepler", "quad_solution_vector"]

from itertools import chain

import aesara_theano_fallback.tensor as tt
import numpy as np
from aesara_theano_fallback import aesara
from aesara_theano_fallback.graph import basic, op

from .. import driver


def as_tensor_variable(x, dtype="float64", **kwargs):
    t = tt.as_tensor_variable(x, **kwargs)
    if dtype is None:
        return t
    return t.astype(dtype)


def resize_or_set(outputs, n, shape, dtype=np.float64):
    if outputs[n][0] is None:
        outputs[n][0] = np.empty(shape, dtype=dtype)
    else:
        outputs[n][0] = np.ascontiguousarray(
            np.resize(outputs[n][0], shape), dtype=dtype
        )
    return outputs[n][0]


# **********
# * KEPLER *
# **********
class Kepler(op.Op):
    r"""Solve Kepler's equation

    This op numerically evaluates the solution to Kepler's equation for given
    mean anomaly ``M`` and eccentricity ``e``:

    .. math::

        M = E - e sin(E)

    For computational efficiency this op actually returns ``cos(f)`` and
    ``sin(f)``, where ``f`` is the true anomaly, defined as

    .. math::

        f = 2\,\arctan\left(\sqrt{\frac{1 + e}{1 - e}}\,\tan\frac{E}{2}\right)

    Args:
        mean_anomaly: The mean anomaly
        eccentricity: Eccentricity, like it says on the box

    Returns:
        (cos(f), sin(f)): The cosine and sine of the true anomaly ``f``.

    """
    __props__ = ()

    def make_node(self, M, ecc):
        in_args = [as_tensor_variable(M), as_tensor_variable(ecc)]
        if any(i.dtype != "float64" for i in in_args):
            raise ValueError("float64 precision is required")
        out_args = [in_args[0].type(), in_args[0].type()]
        return basic.Apply(self, in_args, out_args)

    def infer_shape(self, *args):
        return args[-1]

    def perform(self, node, inputs, outputs):
        M, ecc = inputs
        sinf = resize_or_set(outputs, 0, M.shape)
        cosf = resize_or_set(outputs, 1, M.shape)
        driver.solve_kepler(M, ecc, sinf, cosf)

    def grad(self, inputs, gradients):
        M, e = inputs
        sinf, cosf = self(M, e)

        ecosf = e * cosf
        ome2 = 1 - e ** 2
        dfdM = (1 + ecosf) ** 2 / ome2 ** 1.5
        dfde = (2 + ecosf) * sinf / ome2

        bM = tt.zeros_like(M)
        be = tt.zeros_like(M)
        if not isinstance(gradients[0].type, aesara.gradient.DisconnectedType):
            bM += gradients[0] * cosf * dfdM
            be += gradients[0] * cosf * dfde

        if not isinstance(gradients[1].type, aesara.gradient.DisconnectedType):
            bM -= gradients[1] * sinf * dfdM
            be -= gradients[1] * sinf * dfde

        return [bM, be]

    def R_op(self, inputs, eval_points):
        if eval_points[0] is None:
            return eval_points
        return self.grad(inputs, eval_points)


kepler = Kepler()


# **********
# * STARRY *
# **********
class QuadSolutionVector(op.Op):
    r"""Compute the "solution vector" for a quadratic limb-darkening model

    Note that you probably don't ever want to directly instantiate this op.
    Use ``exoplanet_core.pymc.ops.quad_solution_vector`` instead.

    This will return a tensor with an extra dimension of size 3 on the right
    hand side which represents the solution vector for each pair of impact
    parameter ``b`` and radius ratio ``r`` values. See `Agol+ (2020)
    <https://arxiv.org/abs/1908.03222>`_ for more details.

    Args:
        b: The impact parameter
        r: The radius ratio

    Returns:
        (s, dsdb, dsdr): The solution vector and its first derivatives. Each
            will have the shape ``[..., 3]``, where ``...`` indicates the shape
            of ``b`` or ``r``.

    """
    __props__ = ()

    def make_node(self, b, r):
        in_args = [as_tensor_variable(b), as_tensor_variable(r)]
        if any(i.dtype != "float64" for i in in_args):
            raise ValueError("float64 precision is required")
        x = in_args[0]
        o = [
            tt.tensor(
                broadcastable=tuple(x.broadcastable) + (False,),
                dtype=x.dtype,
            )
            for _ in range(3)
        ]
        return basic.Apply(self, in_args, o)

    def infer_shape(self, *args):
        shapes = args[-1]
        shape = tuple(shapes[0]) + (3,)
        return shape, shape, shape

    def perform(self, node, inputs, outputs):
        b, r = inputs
        shape = b.shape + (3,)
        s = resize_or_set(outputs, 0, shape)
        dsdb = resize_or_set(outputs, 1, shape)
        dsdr = resize_or_set(outputs, 2, shape)
        driver.quad_solution_vector_with_grad(b, r, s, dsdb, dsdr)

    def grad(self, inputs, gradients):
        b, r = inputs
        s, dsdb, dsdr = self(b, r)
        bs = gradients[0]

        for g in gradients[1:]:
            if not isinstance(g.type, aesara.gradient.DisconnectedType):
                raise ValueError(
                    "Backpropagation is only supported for the solution vector"
                )

        if isinstance(bs.type, aesara.gradient.DisconnectedType):
            return [
                aesara.gradient.DisconnectedType()(),
                aesara.gradient.DisconnectedType()(),
            ]

        return [tt.sum(bs * dsdb, axis=-1), tt.sum(bs * dsdr, axis=-1)]

    def R_op(self, inputs, eval_points):
        if eval_points[0] is None:
            return eval_points
        return self.grad(inputs, eval_points)


_quad_solution_vector = QuadSolutionVector()


def quad_solution_vector(b, r):
    return _quad_solution_vector(b, r)[0]


# ******************
# * CONTACT POINTS *
# ******************
class ContactPoints(op.Op):
    __props__ = ()

    def make_node(self, a, e, cosw, sinw, cosi, sini, L):
        in_args = list(
            map(as_tensor_variable, (a, e, cosw, sinw, cosi, sini, L))
        )
        if any(i.dtype != "float64" for i in in_args):
            raise ValueError("float64 precision is required")
        out_args = [
            in_args[0].type(),
            in_args[0].type(),
            tt.tensor(
                broadcastable=tuple(in_args[0].broadcastable),
                dtype="int32",
            ),
        ]
        return basic.Apply(self, in_args, out_args)

    def infer_shape(self, *args):
        shapes = args[-1]
        return (shapes[0], shapes[0], shapes[0])

    def perform(self, node, inputs, outputs):
        shape = inputs[0].shape
        M_left = resize_or_set(outputs, 0, shape)
        M_right = resize_or_set(outputs, 1, shape)
        flag = resize_or_set(outputs, 2, shape, dtype=np.int32)
        driver.contact_points(*chain(inputs, (M_left, M_right, flag)))


contact_points = ContactPoints()



# -*- coding: utf-8 -*-

__all__ = ["__version__", "quad_limbdark_light_curve", "kepler"]

from .core import kepler, quad_limbdark_light_curve
from .exoplanet_core_version import version as __version__

__uri__ = "https://docs.exoplanet.codes"
__author__ = "Daniel Foreman-Mackey"
__email__ = "foreman.mackey@gmail.com"
__license__ = "MIT"
__description__ = "The compiled backend for exoplanet"



#!/usr/bin/env python

# Inspired by:
# https://hynek.me/articles/sharing-your-labor-of-love-pypi-quick-and-dirty/

import codecs
import os
import platform
import re
import subprocess
import sys

import pybind11
from pybind11.setup_helpers import Pybind11Extension, build_ext
from setuptools import find_packages, setup

# PROJECT SPECIFIC

NAME = "exoplanet_core"
PACKAGES = find_packages(where="src")
META_PATH = os.path.join("src", "exoplanet_core", "__init__.py")
CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Intended Audience :: Developers",
    "Intended Audience :: Science/Research",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
]
INSTALL_REQUIRES = ["numpy>=1.13.0"]
EXTRA_REQUIRE = {
    "pymc": ["pymc3>=3.9", "aesara-theano-fallback>=0.0.2"],
    "jax": ["jax", "jaxlib"],
    "test": ["pytest", "pytest-cov>=2.6.1"],
    "benchmark": [
        "pytest",
        "pytest-benchmark",
        "radvel",
        "kepler.py",
        "batman-package",
        "starry",
        "exoplanet==0.4.5",
    ],
}


# END PROJECT SPECIFIC

# CMAKE INTERFACE


def run_cmake(build_temp, install_dir, debug=False):
    import distutils.sysconfig
    from distutils import log

    # From PyTorch
    if platform.system() == "Windows":
        cmake_python_library = "{}/libs/python{}.lib".format(
            distutils.sysconfig.get_config_var("prefix"),
            distutils.sysconfig.get_config_var("VERSION"),
        )
        # Fix virtualenv builds
        if not os.path.exists(cmake_python_library):
            cmake_python_library = "{}/libs/python{}.lib".format(
                sys.base_prefix,
                distutils.sysconfig.get_config_var("VERSION"),
            )
    else:
        cmake_python_library = "{}/{}".format(
            distutils.sysconfig.get_config_var("LIBDIR"),
            distutils.sysconfig.get_config_var("INSTSONAME"),
        )
    cmake_python_include_dir = distutils.sysconfig.get_python_inc()

    cmake_args = [
        "-DCMAKE_INSTALL_PREFIX={}".format(os.path.dirname(install_dir)),
        "-DCMAKE_LIBRARY_OUTPUT_DIRECTORY={}".format(install_dir),
        "-DPython_EXECUTABLE={}".format(sys.executable),
        "-DPython_LIBRARIES={}".format(cmake_python_library),
        "-DPython_INCLUDE_DIRS={}".format(cmake_python_include_dir),
        "-DCMAKE_BUILD_TYPE={}".format("Debug" if debug else "Release"),
        "-DCMAKE_PREFIX_PATH={}".format(pybind11.get_cmake_dir()),
    ]
    build_args = []

    if not os.path.exists(build_temp):
        os.makedirs(build_temp)
    log.info("cmake " + HERE + " " + " ".join(cmake_args))
    subprocess.check_call(["cmake", HERE] + cmake_args, cwd=build_temp)
    log.info("cmake --build . --target install " + " ".join(build_args))
    subprocess.check_call(
        ["cmake", "--build", ".", "--target", "install"] + build_args,
        cwd=build_temp,
    )


class DummyExtension(Pybind11Extension):
    pass


class BuildExt(build_ext):
    def build_extension(self, ext):
        if isinstance(ext, DummyExtension):
            extdir = os.path.abspath(
                os.path.dirname(self.get_ext_fullpath(ext.name))
            )
            run_cmake(self.build_temp, extdir, debug=self.debug)
        else:
            build_ext.build_extension(self, ext)


include_dirs = ["src/exoplanet_core/lib/include"]
ext_modules = [
    Pybind11Extension(
        "exoplanet_core.driver",
        ["src/exoplanet_core/driver.cpp"],
        include_dirs=include_dirs,
        language="c++",
    ),
    Pybind11Extension(
        "exoplanet_core.jax.cpu_driver",
        ["src/exoplanet_core/jax/cpu_driver.cpp"],
        include_dirs=include_dirs + ["src/exoplanet_core/jax"],
        language="c++",
    ),
]

if os.environ.get("EXOPLANET_CORE_CUDA", "no").lower() == "yes":
    ext_modules.append(
        DummyExtension(
            "exoplanet_core.jax.gpu_driver",
            [
                "src/exoplanet_core/jax/gpu_driver.cpp",
                "src/exoplanet_core/jax/cuda_kernels.cc.cu",
            ],
            language="c++",
        )
    )

# END CMAKE INTERFACE

HERE = os.path.dirname(os.path.realpath(__file__))


def read(*parts):
    with codecs.open(os.path.join(HERE, *parts), "rb", "utf-8") as f:
        return f.read()


def find_meta(meta, meta_file=read(META_PATH)):
    meta_match = re.search(
        r"^__{meta}__ = ['\"]([^'\"]*)['\"]".format(meta=meta), meta_file, re.M
    )
    if meta_match:
        return meta_match.group(1)
    raise RuntimeError("Unable to find __{meta}__ string.".format(meta=meta))


if __name__ == "__main__":
    setup(
        name=NAME,
        author=find_meta("author"),
        author_email=find_meta("email"),
        maintainer=find_meta("author"),
        maintainer_email=find_meta("email"),
        url=find_meta("uri"),
        license=find_meta("license"),
        description=find_meta("description"),
        long_description=read("README.md"),
        long_description_content_type="text/markdown",
        packages=PACKAGES,
        package_dir={"": "src"},
        include_package_data=True,
        install_requires=INSTALL_REQUIRES,
        extras_require=EXTRA_REQUIRE,
        classifiers=CLASSIFIERS,
        zip_safe=False,
        ext_modules=ext_modules,
        cmdclass={"build_ext": BuildExt},
    )



# -*- coding: utf-8 -*-

import numpy as np
import pytest

from exoplanet_core.numpy import ops
from exoplanet_core.testing import (
    check_contact_points,
    get_mean_and_true_anomaly,
)


def check_kepler(e, E):
    M, f = get_mean_and_true_anomaly(e, E)
    sinf, cosf = ops.kepler(M, e)
    assert np.all(np.isfinite(cosf))
    np.testing.assert_allclose(cosf, np.cos(f), atol=1e-7)
    assert np.all(np.isfinite(sinf))
    np.testing.assert_allclose(sinf, np.sin(f), atol=1e-7)


def test_kepler():
    e = np.linspace(0, 1, 500)[:-1]
    E = np.linspace(-300, 300, 1001)
    e = e[None, :] + np.zeros((len(E), len(e)))
    E = E[:, None] + np.zeros_like(e)
    check_kepler(e, E)


def test_kepler_edge():
    E = np.array([0.0, 2 * np.pi, -226.2, -170.4])
    e = (1 - 1e-6) * np.ones_like(E)
    e[-1] = 0.9939879759519037
    check_kepler(e, E)

    e = np.linspace(0, 1.0, 100)[:-1]
    E = np.pi + np.zeros_like(e)
    check_kepler(e, E)

    E = 2 * np.pi + np.zeros_like(e)
    check_kepler(e, E)


@pytest.mark.parametrize("a", [5.0, 12.1234, 100.0, 1000.0, 20000.0])
@pytest.mark.parametrize("L", [0.7, 0.9, 1.0, 1.1, 1.5])
def test_contact_points(a, L):
    es = np.linspace(0, 1, 25)[:-1]
    ws = np.linspace(-np.pi, np.pi, 51)
    bs = np.linspace(0, 1 - 1e-5, 5)
    for bi, b in enumerate(bs):
        for ei, e in enumerate(es):
            for wi, w in enumerate(ws):
                check_contact_points(ops.contact_points, L, a, b, e, w)


def test_contact_points_shape_exception():
    args = [
        np.array(16.38049484),
        np.array([0.0002, 0.0002]),
        np.array([0.70710678, 0.70710678]),
        np.array([0.70710678, 0.70710678]),
        np.array([0.04579264, 0.04579264]),
        np.array([0.99895097, 0.99895097]),
        np.array([1.51741758]),
    ]

    with pytest.raises(ValueError):
        ops.contact_points(*args)
